# Generated by roxygen2: do not edit by hand

S3method(as.data.frame,dcvar_model_fit)
S3method(coef,dcvar_constant_fit)
S3method(coef,dcvar_covariate_fit)
S3method(coef,dcvar_fit)
S3method(coef,dcvar_hmm_fit)
S3method(coef,dcvar_multilevel_fit)
S3method(coef,dcvar_multilevel_tv_fit)
S3method(coef,dcvar_sem_fit)
S3method(coef,dcvar_sem_tv_fit)
S3method(coef,dcvar_tv_fit)
S3method(covariate_effects,dcvar_covariate_fit)
S3method(covariate_effects,default)
S3method(dcvar_diagnostics,dcvar_model_fit)
S3method(dcvar_diagnostics,default)
S3method(dependence_summary,dcvar_constant_fit)
S3method(dependence_summary,dcvar_covariate_fit)
S3method(dependence_summary,dcvar_fit)
S3method(dependence_summary,dcvar_hmm_fit)
S3method(dependence_summary,dcvar_multilevel_fit)
S3method(dependence_summary,dcvar_multilevel_tv_fit)
S3method(dependence_summary,dcvar_sem_fit)
S3method(dependence_summary,dcvar_sem_tv_fit)
S3method(dependence_summary,default)
S3method(draws,dcvar_model_fit)
S3method(draws,default)
S3method(fitted,dcvar_hmm_fit)
S3method(fitted,dcvar_model_fit)
S3method(fitted,dcvar_multilevel_fit)
S3method(fitted,dcvar_multilevel_tv_fit)
S3method(fitted,dcvar_sem_fit)
S3method(fitted,dcvar_tv_fit)
S3method(hmm_state_params,dcvar_hmm_fit)
S3method(hmm_state_params,default)
S3method(hmm_states,dcvar_hmm_fit)
S3method(hmm_states,default)
S3method(latent_states,dcvar_sem_fit)
S3method(latent_states,default)
S3method(loo,dcvar_constant_fit)
S3method(loo,dcvar_covariate_fit)
S3method(loo,dcvar_fit)
S3method(loo,dcvar_hmm_fit)
S3method(loo,dcvar_multilevel_fit)
S3method(loo,dcvar_sem_fit)
S3method(phi_trajectory,dcvar_multilevel_tv_fit)
S3method(phi_trajectory,dcvar_sem_tv_fit)
S3method(phi_trajectory,dcvar_tv_fit)
S3method(phi_trajectory,default)
S3method(pit_test,dcvar_model_fit)
S3method(pit_test,dcvar_multilevel_fit)
S3method(pit_test,dcvar_sem_fit)
S3method(pit_test,default)
S3method(pit_values,dcvar_hmm_fit)
S3method(pit_values,dcvar_model_fit)
S3method(pit_values,dcvar_multilevel_fit)
S3method(pit_values,dcvar_sem_fit)
S3method(pit_values,dcvar_tv_fit)
S3method(pit_values,default)
S3method(plot,dcvar_constant_fit)
S3method(plot,dcvar_covariate_fit)
S3method(plot,dcvar_fit)
S3method(plot,dcvar_hmm_fit)
S3method(plot,dcvar_multilevel_fit)
S3method(plot,dcvar_sem_fit)
S3method(plot,dcvar_tv_fit)
S3method(predict,dcvar_hmm_fit)
S3method(predict,dcvar_model_fit)
S3method(predict,dcvar_multilevel_fit)
S3method(predict,dcvar_multilevel_tv_fit)
S3method(predict,dcvar_sem_fit)
S3method(predict,dcvar_tv_fit)
S3method(print,dcvar_applicability)
S3method(print,dcvar_constant_fit)
S3method(print,dcvar_constant_summary)
S3method(print,dcvar_covariate_fit)
S3method(print,dcvar_covariate_summary)
S3method(print,dcvar_fit)
S3method(print,dcvar_hmm_fit)
S3method(print,dcvar_hmm_summary)
S3method(print,dcvar_multilevel_fit)
S3method(print,dcvar_multilevel_summary)
S3method(print,dcvar_multilevel_tv_fit)
S3method(print,dcvar_multilevel_tv_summary)
S3method(print,dcvar_sem_fit)
S3method(print,dcvar_sem_summary)
S3method(print,dcvar_sem_tv_fit)
S3method(print,dcvar_sem_tv_summary)
S3method(print,dcvar_summary)
S3method(print,dcvar_tv_fit)
S3method(print,dcvar_tv_summary)
S3method(random_effects,dcvar_multilevel_fit)
S3method(random_effects,default)
S3method(rho_trajectory,dcvar_constant_fit)
S3method(rho_trajectory,dcvar_covariate_fit)
S3method(rho_trajectory,dcvar_fit)
S3method(rho_trajectory,dcvar_hmm_fit)
S3method(rho_trajectory,dcvar_multilevel_fit)
S3method(rho_trajectory,dcvar_multilevel_tv_fit)
S3method(rho_trajectory,dcvar_sem_fit)
S3method(rho_trajectory,dcvar_sem_tv_fit)
S3method(rho_trajectory,default)
S3method(sigma_trajectory,dcvar_multilevel_tv_fit)
S3method(sigma_trajectory,dcvar_sem_tv_fit)
S3method(sigma_trajectory,dcvar_tv_fit)
S3method(sigma_trajectory,default)
S3method(summary,dcvar_constant_fit)
S3method(summary,dcvar_covariate_fit)
S3method(summary,dcvar_fit)
S3method(summary,dcvar_hmm_fit)
S3method(summary,dcvar_multilevel_fit)
S3method(summary,dcvar_multilevel_tv_fit)
S3method(summary,dcvar_sem_fit)
S3method(summary,dcvar_sem_tv_fit)
S3method(summary,dcvar_tv_fit)
S3method(var_params,dcvar_hmm_fit)
S3method(var_params,dcvar_model_fit)
S3method(var_params,dcvar_multilevel_fit)
S3method(var_params,dcvar_multilevel_tv_fit)
S3method(var_params,dcvar_sem_fit)
S3method(var_params,dcvar_sem_tv_fit)
S3method(var_params,dcvar_tv_fit)
S3method(var_params,default)
export(aggregate_metrics)
export(applicability_check)
export(compute_param_metrics)
export(compute_rho_metrics)
export(covariate_effects)
export(dcvar)
export(dcvar_compare)
export(dcvar_constant)
export(dcvar_covariate)
export(dcvar_diagnostics)
export(dcvar_hmm)
export(dcvar_multilevel)
export(dcvar_sem)
export(dcvar_stan_path)
export(dependence_summary)
export(draws)
export(hmm_state_params)
export(hmm_states)
export(interpret_rho_trajectory)
export(latent_states)
export(phi_trajectory)
export(pit_test)
export(pit_values)
export(plot_diagnostics)
export(plot_hmm_states)
export(plot_latent_states)
export(plot_phi)
export(plot_phi_trajectory)
export(plot_pit)
export(plot_ppc)
export(plot_random_effects)
export(plot_rho)
export(plot_sigma_trajectory)
export(plot_trajectories)
export(prepare_constant_data)
export(prepare_dcvar_covariate_data)
export(prepare_dcvar_data)
export(prepare_hmm_data)
export(prepare_multilevel_data)
export(prepare_sem_data)
export(random_effects)
export(rho_constant)
export(rho_decreasing)
export(rho_double_step)
export(rho_increasing)
export(rho_random_walk)
export(rho_scenario)
export(rho_step)
export(rho_trajectory)
export(sigma_trajectory)
export(simulate_breakpoint_data)
export(simulate_dcvar)
export(simulate_dcvar_multilevel)
export(simulate_dcvar_sem)
export(var_params)
importFrom(cli,cli_abort)
importFrom(cli,cli_alert_info)
importFrom(cli,cli_alert_success)
importFrom(cli,cli_inform)
importFrom(cli,cli_warn)
importFrom(loo,loo)
importFrom(rlang,"%||%")
importFrom(rlang,.data)
importFrom(rstan,sampling)
importFrom(rstan,stan_model)
importFrom(stats,coef)
importFrom(stats,complete.cases)
importFrom(stats,cor)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(utils,head)
