% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extract.R
\name{draws}
\alias{draws}
\alias{draws.default}
\alias{draws.dcvar_model_fit}
\title{Extract posterior draws}
\usage{
draws(object, ...)

\method{draws}{default}(object, ...)

\method{draws}{dcvar_model_fit}(object, variable = NULL, format = "draws_array", ...)
}
\arguments{
\item{object}{A fitted model object.}

\item{...}{Additional arguments (unused).}

\item{variable}{Character vector of parameter names. \code{NULL} returns all.}

\item{format}{Draw format: \code{"draws_array"}, \code{"draws_matrix"}, or
\code{"draws_df"} (default: \code{"draws_array"}).}
}
\value{
A posterior draws object.
}
\description{
Extract posterior draws from a fitted model.
}
