% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/applicability.R
\name{print.dcvar_applicability}
\alias{print.dcvar_applicability}
\title{Print a flexible-margin applicability check}
\usage{
\method{print}{dcvar_applicability}(x, ...)
}
\arguments{
\item{x}{A \code{dcvar_applicability} object, as returned by
\code{\link[=applicability_check]{applicability_check()}}.}

\item{...}{Additional arguments (unused).}
}
\value{
Invisibly returns \code{x}.
}
\description{
Print a flexible-margin applicability check
}
