% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plot_random_effects}
\alias{plot_random_effects}
\title{Plot random effects (caterpillar plot)}
\usage{
plot_random_effects(object, ...)
}
\arguments{
\item{object}{A \code{dcvar_multilevel_fit} object.}

\item{...}{Additional arguments (unused).}
}
\value{
A ggplot object.
}
\description{
Displays unit-specific VAR coefficients with credible intervals.
}
